###################################################
### chunk number 1: set-opts
###################################################
#line 58 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
  glop <- options(keep.source=TRUE,width=60,continue=" ",prompt=" ")
  library(pomp)
  pdf.options(useDingbats=FALSE)
  set.seed(5384959)


###################################################
### chunk number 2: plugin-R-code
###################################################
#line 98 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
data(ou2)
ou2.dat <- as.data.frame(ou2)

pomp( 
     data=ou2.dat[c("time","y1","y2")],
     times="time",
     t0=0,
     rprocess=discrete.time.sim(
       step.fun=function (x, t, params, ...) {
         eps <- rnorm(n=2,mean=0,sd=1) # noise terms
         xnew <- c(
                   x1=params["alpha.1"]*x["x1"]+params["alpha.3"]*x["x2"]+
                       params["sigma.1"]*eps[1],
                   x2=params["alpha.2"]*x["x1"]+params["alpha.4"]*x["x2"]+
                       params["sigma.2"]*eps[1]+params["sigma.3"]*eps[2]
                   )
         names(xnew) <- c("x1","x2")
         xnew
       }
       )
     ) -> ou2.Rplug


###################################################
### chunk number 3: plugin-R-code-sim
###################################################
#line 121 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
tic <- Sys.time()
simdat.Rplug <- simulate(ou2.Rplug,params=coef(ou2),nsim=1000,states=T)
toc <- Sys.time()
etime.Rplug <- toc-tic


###################################################
### chunk number 4: vectorized-R-code
###################################################
#line 149 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
ou2.Rvect.rprocess <- function (xstart, times, params, ...) {
  nrep <- ncol(xstart)                  # number of realizations
  ntimes <- length(times)               # number of timepoints
  ## unpack the parameters (for legibility only)
  alpha.1 <- params["alpha.1",]
  alpha.2 <- params["alpha.2",]
  alpha.3 <- params["alpha.3",]
  alpha.4 <- params["alpha.4",]
  sigma.1 <- params["sigma.1",]
  sigma.2 <- params["sigma.2",]
  sigma.3 <- params["sigma.3",]
  ## x is the array of states to be returned: it must have rownames
  x <- array(0,dim=c(2,nrep,ntimes))
  rownames(x) <- rownames(xstart)
  ## xnow holds the current state values
  x[,,1] <- xnow <- xstart
  tnow <- times[1]
  for (k in seq.int(from=2,to=ntimes,by=1)) {
    tgoal <- times[k]
    while (tnow < tgoal) {              # take one step at a time
      eps <- array(rnorm(n=2*nrep,mean=0,sd=1),dim=c(2,nrep))
      tmp <- alpha.1*xnow['x1',]+alpha.3*xnow['x2',]+
        sigma.1*eps[1,]
      xnow['x2',] <- alpha.2*xnow['x1',]+alpha.4*xnow['x2',]+
        sigma.2*eps[1,]+sigma.3*eps[2,]
      xnow['x1',] <- tmp
      tnow <- tnow+1
    }
    x[,,k] <- xnow
  }
  x
}


###################################################
### chunk number 5: vectorized-R-pomp
###################################################
#line 184 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
ou2.Rvect <- pomp(ou2.Rplug,rprocess=ou2.Rvect.rprocess)


###################################################
### chunk number 6: theta
###################################################
#line 189 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
theta <- c(
           x1.0=-3, x2.0=4,
           tau=1,
           alpha.1=0.8, alpha.2=-0.5, alpha.3=0.3, alpha.4=0.9,
           sigma.1=3, sigma.2=-0.5, sigma.3=2
           )


###################################################
### chunk number 7: vectorized-R-code-sim
###################################################
#line 197 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
tic <- Sys.time()
simdat.Rvect <- simulate(ou2.Rvect,params=theta,states=T,nsim=1000)
toc <- Sys.time()
etime.Rvect <- toc-tic


###################################################
### chunk number 8: 
###################################################
#line 203 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
units(etime.Rvect) <- units(etime.Rplug)


###################################################
### chunk number 9: plugin-byte-code
###################################################
#line 213 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
require(compiler)
pomp( 
     ou2.Rplug,
     rprocess=discrete.time.sim(
       step.fun=cmpfun(
         function (x, t, params, ...) {
           eps <- rnorm(n=2,mean=0,sd=1) # noise terms
           xnew <- c(
                     x1=params["alpha.1"]*x["x1"]+params["alpha.3"]*x["x2"]+
                     params["sigma.1"]*eps[1],
                     x2=params["alpha.2"]*x["x1"]+params["alpha.4"]*x["x2"]+
                     params["sigma.2"]*eps[1]+params["sigma.3"]*eps[2]
                     )
           names(xnew) <- c("x1","x2")
           xnew
         },
         options=list(optimize=3)
         )
       )
     ) -> ou2.Bplug


###################################################
### chunk number 10: plugin-byte-code-sim
###################################################
#line 235 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
tic <- Sys.time()
simdat.Bplug <- simulate(ou2.Bplug,params=coef(ou2),nsim=1000,states=T)
toc <- Sys.time()
etime.Bplug <- toc-tic


###################################################
### chunk number 11: vectorized-byte-code
###################################################
#line 245 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
ou2.Bvect <- pomp(ou2.Rplug,rprocess=cmpfun(ou2.Rvect.rprocess,options=list(optimize=3)))


###################################################
### chunk number 12: vectorized-byte-code-sim
###################################################
#line 248 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
tic <- Sys.time()
simdat.Bvect <- simulate(ou2.Bvect,params=theta,states=T,nsim=1000)
toc <- Sys.time()
etime.Bvect <- toc-tic


###################################################
### chunk number 13: 
###################################################
#line 254 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
units(etime.Bvect) <- units(etime.Rplug)


###################################################
### chunk number 14: view-ou2-source eval=FALSE
###################################################
## #line 264 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
## file.show(file=system.file("examples/ou2.c",package="pomp"))


###################################################
### chunk number 15: view-pomp.h eval=FALSE
###################################################
## #line 270 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
## file.show(file=system.file("include/pomp.h",package="pomp"))


###################################################
### chunk number 16: plugin-C-code
###################################################
#line 274 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
ou2.Cplug <- pomp(
                  ou2.Rplug,
                  rprocess=discrete.time.sim("ou2_step"),
                  paramnames=c(
                    "alpha.1","alpha.2","alpha.3","alpha.4",
                    "sigma.1","sigma.2","sigma.3",
                    "tau"
                    ),
                  statenames=c("x1","x2"),
                  obsnames=c("y1","y2")
                  )


###################################################
### chunk number 17: plugin-C-sim
###################################################
#line 287 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
tic <- Sys.time()
simdat.Cplug <- simulate(ou2.Cplug,params=theta,states=T,nsim=100000)
toc <- Sys.time()
etime.Cplug <- toc-tic


###################################################
### chunk number 18: 
###################################################
#line 295 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
units(etime.Cplug) <- units(etime.Rplug)
speedup <- as.numeric(etime.Rplug/dim(simdat.Rplug)[2])/as.numeric(etime.Cplug/dim(simdat.Cplug)[2])


###################################################
### chunk number 19:  eval=FALSE
###################################################
## #line 306 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
## #line 264 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw#from line#306#"
## file.show(file=system.file("examples/ou2.c",package="pomp"))
## #line 307 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"


###################################################
### chunk number 20: vectorized-C-code
###################################################
#line 311 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
ou2.Cvect.rprocess <- function (xstart, times, params, ...) {
  nvar <- nrow(xstart)
  npar <- nrow(params)
  nrep <- ncol(xstart)
  ntimes <- length(times)
  array(
        .C("ou2_adv",
           X = double(nvar*nrep*ntimes),
           xstart = as.double(xstart),
           par = as.double(params),
           times = as.double(times),
           n = as.integer(c(nvar,npar,nrep,ntimes))
           )$X,
        dim=c(nvar,nrep,ntimes),
        dimnames=list(rownames(xstart),NULL,NULL)        
        )
}


###################################################
### chunk number 21: vectorized-C-code-pomp
###################################################
#line 332 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
ou2.Cvect <- pomp(
                  ou2.Rplug,
                  rprocess=ou2.Cvect.rprocess
                  )


###################################################
### chunk number 22: vectorized-C-code-sim
###################################################
#line 338 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
tic <- Sys.time()
paramnames <- c(
                "alpha.1","alpha.2","alpha.3","alpha.4",
                "sigma.1","sigma.2","sigma.3",
                "tau",
                "x1.0","x2.0"
                )
simdat.Cvect <- simulate(ou2.Cvect,params=theta[paramnames],nsim=100000,states=T)
toc <- Sys.time()
etime.Cvect <- toc-tic


###################################################
### chunk number 23: 
###################################################
#line 351 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
units(etime.Cvect) <- units(etime.Rplug)
speedup <- as.numeric(etime.Rplug/dim(simdat.Rplug)[2])/as.numeric(etime.Cvect/dim(simdat.Cvect)[2])


###################################################
### chunk number 24: sir-def
###################################################
#line 363 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
pomp(
     data=data.frame(
       time=seq(from=1/52,to=4,by=1/52),
       reports=NA
       ),
     times="time",
     t0=0,
     ## native routine for the process simulator:
     rprocess=euler.sim(
       step.fun="_sir_euler_simulator", 
       delta.t=1/52/20,
       PACKAGE="pomp"
       ),
     ## native routine for the skeleton:
     skeleton.type="vectorfield",
     skeleton="_sir_ODE", 
     ## binomial measurement model:
     rmeasure="_sir_binom_rmeasure", 
     dmeasure="_sir_binom_dmeasure", 
     ## name of the shared-object library containing the 
     ## native measurement-model routines:
     PACKAGE="pomp", 
     ## the order of the observable assumed in the native routines:
     obsnames=c("reports"),
     ## the order of the state variables assumed in the native routines:
     statenames=c("S","I","R","cases","W"),
     ## the order of the parameters assumed in the native routines:
     paramnames=c(
       "gamma","mu","iota","beta1","beta.sd",
       "pop","rho","nbasis","degree","period"
       ), 
     ## reset cases to zero at each new observation:
     zeronames=c("cases"),      
     comp.names=c("S","I","R"),
     to.log.transform=c(
       "gamma","mu","iota",
       "beta1","beta2","beta3","beta.sd",
       "rho",
       "S.0","I.0","R.0"
       ),
     parameter.transform=function (params, to.log.transform, ...) {
       params[to.log.transform] <- log(params[to.log.transform])
       params
     },
     parameter.inv.transform=function (params, to.log.transform, ...) {
       params[to.log.transform] <- exp(params[to.log.transform])
       params
     },
     initializer=function(params, t0, comp.names, ...) {
       ic.names <- paste(comp.names,".0",sep="")
       snames <- c("S","I","R","cases","W")
       fracs <- exp(params[ic.names])
       x0 <- numeric(length(snames))
       names(x0) <- snames
       x0[comp.names] <- round(params['pop']*fracs/sum(fracs))
       ## since 'cases' is in 'zeronames' above, the IC for this variable
       ## will only matter in trajectory computations
       ## In trajectory computations, however, 'cases' will be roughly the weekly number of new cases
       x0["cases"] <- x0["I"]*exp(params["gamma"])/52 
       x0
     }
     ) -> sir


###################################################
### chunk number 25: view-sir-source eval=FALSE
###################################################
## #line 429 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
## file.show(file=system.file("examples/sir.c",package="pomp"))


###################################################
### chunk number 26: sir-sim
###################################################
#line 452 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
params <- c(
            gamma=26,mu=0.02,iota=0.01,
            beta1=1200,beta2=1800,beta3=600,
            beta.sd=1e-3,                      
            pop=2.1e6,
            rho=0.6,
            S.0=26/1200,I.0=0.001,R.0=1-0.001-26/1200
            )                                                     

sir <- simulate(sir,params=c(log(params),nbasis=3,degree=3,period=1),seed=3493885L)
sims <- simulate(sir,nsim=10,obs=T)
traj <- trajectory(sir,hmax=1/52)


###################################################
### chunk number 27: 
###################################################
#line 478 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
data(ou2)
true.p <- coef(ou2)
x0 <- init.state(ou2)
x0
new.p <- cbind(true.p,true.p,true.p)
new.p["x1.0",] <- 1:3
init.state(ou2,params=new.p)


###################################################
### chunk number 28: 
###################################################
#line 494 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
x <- rprocess(ou2,xstart=x0,times=time(ou2,t0=T),params=as.matrix(true.p))
dim(x)
x[,,1:5]


###################################################
### chunk number 29: 
###################################################
#line 505 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
x <- x[,,-1,drop=F]
y <- rmeasure(ou2,x=x,times=time(ou2),params=as.matrix(true.p))
dim(y)
y[,,1:5]


###################################################
### chunk number 30: 
###################################################
#line 515 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
fp <- dprocess(ou2,x=x,times=time(ou2),params=as.matrix(true.p))
dim(fp)
fp[,36:40]


###################################################
### chunk number 31: 
###################################################
#line 520 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
fm <- dmeasure(ou2,y=y[,1,],x=x,times=time(ou2),params=as.matrix(true.p))
dim(fm)
fm[,36:40]


###################################################
### chunk number 32: all-data-loaable eval=FALSE
###################################################
## #line 531 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
## data(package="pomp")


###################################################
### chunk number 33: restore-opts
###################################################
#line 537 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/advanced_topics_in_pomp.Rnw"
options(glop)


