###################################################
### chunk number 1: set-opts
###################################################
#line 65 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  glop <- options(keep.source=TRUE,width=60,continue=" ",prompt=" ")
  library(pomp)
  pdf.options(useDingbats=FALSE)
  set.seed(5384959)


###################################################
### chunk number 2: gompertz-proc-sim-def
###################################################
#line 199 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
require(pomp)

gompertz.proc.sim <- function (x, t, params, delta.t, ...) {
  ## unpack the parameters:
  r <- params["r"]
  K <- params["K"]
  sigma <- params["sigma"]
  ## the state at time t:
  X <- x["X"]
  ## generate a log-normal random variable:
  eps <- exp(rnorm(n=1,mean=0,sd=sigma))
  ## compute the state at time t+delta.t:
  S <- exp(-r*delta.t)
  xnew <- c(X=unname(K^(1-S)*X^S*eps))
  return(xnew)
}


###################################################
### chunk number 3: gompertz-meas-sim-def
###################################################
#line 228 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
gompertz.meas.sim <- function (x, t, params, ...) {
  ## unpack the parameters:
  tau <- params["tau"]
  ## state at time t:
  X <- x["X"]
  ## generate a simulated observation:
  y <- c(Y=unname(rlnorm(n=1,meanlog=log(X),sd=tau)))
  return(y)
}


###################################################
### chunk number 4: gompertz-meas-dens-def
###################################################
#line 244 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
gompertz.meas.dens <- function (y, x, t, params, log, ...) {
  ## unpack the parameters:
  tau <- params["tau"]
  ## state at time t:
  X <- x["X"]
  ## observation at time t:
  Y <- y["Y"]
  ## compute the likelihood of Y|X,tau
  f <- dlnorm(x=Y,meanlog=log(X),sdlog=tau,log=log)
  return(f)
}


###################################################
### chunk number 5: first-pomp-construction eval=FALSE
###################################################
## #line 266 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## gompertz <- pomp(
##                  data=data.frame(
##                    time=1:100,
##                    Y=NA
##                    ),
##                  times="time",
##                  rprocess=discrete.time.sim(
##                    step.fun=gompertz.proc.sim,
##                    delta.t=1
##                    ),
##                  rmeasure=gompertz.meas.sim,
##                  t0=0
##                  )


###################################################
### chunk number 6: set-params
###################################################
#line 295 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
theta <- c(
           r=0.1,K=1,sigma=0.1,
           tau=0.1,
           X.0=1
           )


###################################################
### chunk number 7: gompertz-first-simulation eval=FALSE
###################################################
## #line 309 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## gompertz <- simulate(gompertz,params=theta)


###################################################
### chunk number 8: gompertz-get-data
###################################################
#line 312 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
data(gompertz)
dat <- as.data.frame(gompertz)
gompertz <- pomp(
                 data=dat[c("time","Y")],
                 times="time",
                 rprocess=discrete.time.sim(
                   step.fun=gompertz.proc.sim,
                   delta.t=1
                   ),
                 rmeasure=gompertz.meas.sim,
                 t0=0
                 )
coef(gompertz) <- theta


###################################################
### chunk number 9:  eval=FALSE
###################################################
## #line 330 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## plot(gompertz,variables="Y")


###################################################
### chunk number 10: gompertz-plot
###################################################
#line 336 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
plot(gompertz,variables=c("Y"))


###################################################
### chunk number 11: second-pomp-construction
###################################################
#line 355 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
gompertz <- pomp(
                 gompertz,
                 dmeasure=gompertz.meas.dens
                 )
coef(gompertz) <- theta


###################################################
### chunk number 12: gompertz-pfilter-truth eval=FALSE
###################################################
## #line 369 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## pf <- pfilter(gompertz,params=theta,Np=1000)
## loglik.truth <- logLik(pf)
## loglik.truth


###################################################
### chunk number 13: gompertz-pfilter-truth-eval
###################################################
#line 374 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
set.seed(457645443L)
#line 369 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#375#"
pf <- pfilter(gompertz,params=theta,Np=1000)
loglik.truth <- logLik(pf)
loglik.truth
#line 376 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"


###################################################
### chunk number 14: gompertz-pfilter-truth-alt1 eval=FALSE
###################################################
## #line 379 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## pf <- pfilter(gompertz,params=coef(gompertz),Np=1000)


###################################################
### chunk number 15: gompertz-pfilter-truth-alt2 eval=FALSE
###################################################
## #line 383 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## pf <- pfilter(gompertz,Np=1000)


###################################################
### chunk number 16: gompertz-pfilter-guess eval=FALSE
###################################################
## #line 388 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## theta.true <- coef(gompertz)
## theta.guess <- theta.true
## theta.guess[c("r","K","sigma")] <- 1.5*theta.true[c("r","K","sigma")]
## pf <- pfilter(gompertz,params=theta.guess,Np=1000)
## loglik.guess <- logLik(pf)


###################################################
### chunk number 17: gompertz-pfilter-guess-eval
###################################################
#line 395 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
set.seed(457645443L)
#line 388 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#396#"
theta.true <- coef(gompertz)
theta.guess <- theta.true
theta.guess[c("r","K","sigma")] <- 1.5*theta.true[c("r","K","sigma")]
pf <- pfilter(gompertz,params=theta.guess,Np=1000)
loglik.guess <- logLik(pf)
#line 397 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"


###################################################
### chunk number 18: kalman-filter-def
###################################################
#line 403 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
kalman.filter <- function (Y, X0, r, K, sigma, tau) {
  ntimes <- length(Y)
  sigma.sq <- sigma^2
  tau.sq <- tau^2
  cond.loglik <- numeric(ntimes)
  filter.mean <- numeric(ntimes)
  pred.mean <- numeric(ntimes)
  pred.var <- numeric(ntimes)
  m <- log(X0)
  v <- 0
  S <- exp(-r)
  for (k in seq_len(ntimes)) {
    pred.mean[k] <- M <- (1-S)*log(K) + S*m
    pred.var[k] <- V <- S*v*S+sigma.sq
    q <- V+tau.sq
    r <- log(Y[k])-M
    cond.loglik[k] <- dnorm(x=log(Y[k]),mean=M,sd=sqrt(q),log=TRUE)
    q <- 1/V+1/tau.sq
    filter.mean[k] <- m <- (log(Y[k])/tau.sq+M/V)/q
    v <- 1/q
  }
  list(
       pred.mean=pred.mean,
       pred.var=pred.var,
       filter.mean=filter.mean,
       cond.loglik=cond.loglik,
       loglik=sum(cond.loglik)
       )
}


###################################################
### chunk number 19: kalman-filter-run
###################################################
#line 439 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
y <- obs(gompertz)
x0 <- init.state(gompertz)
r <- coef(gompertz,"r")
K <- coef(gompertz,"K")
sigma <- coef(gompertz,"sigma")
tau <- coef(gompertz,"tau")
kf <- kalman.filter(y,x0,r,K,sigma,tau)


###################################################
### chunk number 20: kalman-likelihood-correction
###################################################
#line 448 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
loglik.kalman <- kf$loglik-sum(log(obs(gompertz)))


###################################################
### chunk number 21:  eval=FALSE
###################################################
## #line 464 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## as(gompertz,"data.frame")


###################################################
### chunk number 22:  eval=FALSE
###################################################
## #line 469 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## obs(gompertz)
## obs(gompertz,"Y")
## time(gompertz)  


###################################################
### chunk number 23:  eval=FALSE
###################################################
## #line 475 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## time(gompertz) <- 1:10


###################################################
### chunk number 24:  eval=FALSE
###################################################
## #line 479 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## timezero(gompertz)
## timezero(gompertz) <- -10


###################################################
### chunk number 25:  eval=FALSE
###################################################
## #line 484 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## time(gompertz,t0=TRUE)  
## time(gompertz,t0=T) <- seq(from=0,to=10,by=1)


###################################################
### chunk number 26:  eval=FALSE
###################################################
## #line 489 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## window(gompertz,start=3,end=20)


###################################################
### chunk number 27:  eval=FALSE
###################################################
## #line 494 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## coef(gompertz)
## coef(gompertz,c("sigma","tau")) <- c(1,0)


###################################################
### chunk number 28:  eval=FALSE
###################################################
## #line 500 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## states(gompertz)
## states(gompertz,"X")


###################################################
### chunk number 29: loggomp-proc-sim-def
###################################################
#line 516 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
gompertz.proc.sim <- function (x, t, params, delta.t, ...) {
  ## unpack and untransform the parameters:
  r <- exp(params["log.r"])
  K <- exp(params["log.K"])
  sigma <- exp(params["log.sigma"])
  ## the state at time t:
  X <- x["X"]
  ## generate a log-normal random variable:
  eps <- exp(rnorm(n=1,mean=0,sd=sigma))
  ## compute the state at time t+delta.t:
  S <- exp(-r*delta.t)
  xnew <- c(X=unname(K^(1-S)*X^S*eps))
  return(xnew)
}


###################################################
### chunk number 30: loggomp-meas-sim-def
###################################################
#line 533 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
gompertz.meas.sim <- function (x, t, params, ...) {
  ## unpack and untransform the parameters:
  tau <- exp(params["log.tau"])
  ## state at time t:
  X <- x["X"]
  ## generate a simulated observation:
  y <- c(Y=unname(rlnorm(n=1,meanlog=log(X),sdlog=tau)))
  return(y)
}


###################################################
### chunk number 31: loggomp-meas-dens-def
###################################################
#line 545 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
gompertz.meas.dens <- function (y, x, t, params, log, ...) {
  ## unpack and untransform the parameters:
  tau <- exp(params["log.tau"])
  ## state at time t:
  X <- x["X"]
  ## observation at time t:
  Y <- y["Y"]
  ## compute the likelihood of Y|X,tau
  f <- dlnorm(x=Y,meanlog=log(X),sdlog=tau,log=log)
  return(f)
}


###################################################
### chunk number 32: loggomp-pomp-construction
###################################################
#line 566 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
dat <- as(gompertz,"data.frame")
theta <- coef(gompertz)
print(theta)
gompertz <- pomp(
                 data=dat[c("time","Y")],
                 times="time",
                 rprocess=discrete.time.sim(gompertz.proc.sim),
                 rmeasure=gompertz.meas.sim,
                 dmeasure=gompertz.meas.dens,
                 t0=0,
                 parameter.transform=function(params,...){
                   params <- c(
                               params["X.0"],
                               log(params[c("r","K","tau","sigma")])
                               )
                   names(params) <- c("X.0","log.r","log.K","log.tau","log.sigma")
                   params
                 },
                 parameter.inv.transform=function(params,...){
                   params <- c(
                               params["X.0"],
                               exp(params[c("log.r","log.K","log.tau","log.sigma")])
                               )
                   names(params) <- c("X.0","r","K","tau","sigma")
                   params
                 }
                 )


###################################################
### chunk number 33: 
###################################################
#line 598 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
coef(gompertz,transform=TRUE) <- theta


###################################################
### chunk number 34: 
###################################################
#line 602 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
coef(gompertz,transform=TRUE)


###################################################
### chunk number 35: 
###################################################
#line 607 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
coef(gompertz)


###################################################
### chunk number 36: par-trans-inverse-test
###################################################
#line 613 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
# use parameter.inv.transform:
theta <- coef(gompertz,transform=TRUE)  
## theta is on the natural scale
g2 <- gompertz
## use parameter.transform:
coef(g2,transform=TRUE) <- theta
## compare the internal-scale representations:
identical(coef(gompertz),coef(g2))


###################################################
### chunk number 37:  eval=FALSE
###################################################
## #line 627 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## file.show(system.file("examples/gompertz.R",package="pomp"))
## file.show(system.file("examples/gompertz.c",package="pomp"))


###################################################
### chunk number 38: 
###################################################
#line 656 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
data(gompertz)
theta <- coef(gompertz)
theta.true <- theta


###################################################
### chunk number 39: gompertz-multi-mif-calc eval=FALSE
###################################################
## #line 661 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## estpars <- c("log.r","log.sigma","log.tau")
## mf <- replicate(
##                 n=10,
##                 {
##                   theta.guess <- theta.true
##                   theta.guess[estpars] <- rnorm(
##                                                 n=length(estpars),
##                                                 mean=theta.guess[estpars],
##                                                 sd=1
##                                                 )
##                   mif(
##                       gompertz,
##                       Nmif=100,
##                       start=theta.guess,
##                       pars=estpars,
##                       rw.sd=c(
##                         log.r=0.02,log.sigma=0.02,log.tau=0.05
##                         ),
##                       Np=2000,
##                       var.factor=4,
##                       ic.lag=10,
##                       cooling.factor=0.999,
##                       max.fail=10
##                       )
##                 }
##                 )
## ##mf <- lapply(mf,continue,Nmif=50)


###################################################
### chunk number 40: gompertz-post-mif eval=FALSE
###################################################
## #line 691 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## theta.true <- coef(gompertz)
## theta.mif <- apply(sapply(mf,coef),1,mean)
## loglik.mif <- replicate(n=10,logLik(pfilter(mf[[1]],params=theta.mif,Np=10000)))
## bl <- mean(loglik.mif)
## loglik.mif.est <- bl+log(mean(exp(loglik.mif-bl)))
## loglik.mif.se <- sd(exp(loglik.mif-bl))/sqrt(length(loglik.mif))/exp(loglik.mif.est-bl)
## loglik.true <- replicate(n=10,logLik(pfilter(gompertz,params=theta.true,Np=10000)))
## loglik.true.est <- bl+log(mean(exp(loglik.true-bl)))
## loglik.true.se <- sd(exp(loglik.true-bl))/sqrt(length(loglik.true))/exp(loglik.true.est-bl)


###################################################
### chunk number 41: gompertz-multi-mif-eval
###################################################
#line 703 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
set.seed(334388458L)
binary.file <- "gompertz-multi-mif.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
  tic <- Sys.time()
#line 661 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#709#"
estpars <- c("log.r","log.sigma","log.tau")
mf <- replicate(
                n=10,
                {
                  theta.guess <- theta.true
                  theta.guess[estpars] <- rnorm(
                                                n=length(estpars),
                                                mean=theta.guess[estpars],
                                                sd=1
                                                )
                  mif(
                      gompertz,
                      Nmif=100,
                      start=theta.guess,
                      pars=estpars,
                      rw.sd=c(
                        log.r=0.02,log.sigma=0.02,log.tau=0.05
                        ),
                      Np=2000,
                      var.factor=4,
                      ic.lag=10,
                      cooling.factor=0.999,
                      max.fail=10
                      )
                }
                )
##mf <- lapply(mf,continue,Nmif=50)
#line 710 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
#line 691 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#710#"
theta.true <- coef(gompertz)
theta.mif <- apply(sapply(mf,coef),1,mean)
loglik.mif <- replicate(n=10,logLik(pfilter(mf[[1]],params=theta.mif,Np=10000)))
bl <- mean(loglik.mif)
loglik.mif.est <- bl+log(mean(exp(loglik.mif-bl)))
loglik.mif.se <- sd(exp(loglik.mif-bl))/sqrt(length(loglik.mif))/exp(loglik.mif.est-bl)
loglik.true <- replicate(n=10,logLik(pfilter(gompertz,params=theta.true,Np=10000)))
loglik.true.est <- bl+log(mean(exp(loglik.true-bl)))
loglik.true.se <- sd(exp(loglik.true-bl))/sqrt(length(loglik.true))/exp(loglik.true.est-bl)
#line 711 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  toc <- Sys.time()
  etime <- toc-tic
  save(
       mf,estpars,
       theta.mif,theta.true,
       loglik.mif.est,loglik.true.est,
       loglik.mif.se,loglik.true.se,
       etime,
       file=binary.file
       )
}
rbind(
#      guess=c(signif(theta.guess[estpars],3),loglik=round(loglik.guess,1)),
      mle=c(signif(theta.mif[estpars],3),loglik=round(loglik.mif.est,1),loglik.se=signif(loglik.mif.se,2)),
      truth=c(signif(theta.true[estpars],3),loglik=round(loglik.true.est,1),loglik.se=signif(loglik.true.se,2))
      ) -> results.table


###################################################
### chunk number 42:  eval=FALSE
###################################################
## #line 735 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## #line 691 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#735#"
## theta.true <- coef(gompertz)
## theta.mif <- apply(sapply(mf,coef),1,mean)
## loglik.mif <- replicate(n=10,logLik(pfilter(mf[[1]],params=theta.mif,Np=10000)))
## bl <- mean(loglik.mif)
## loglik.mif.est <- bl+log(mean(exp(loglik.mif-bl)))
## loglik.mif.se <- sd(exp(loglik.mif-bl))/sqrt(length(loglik.mif))/exp(loglik.mif.est-bl)
## loglik.true <- replicate(n=10,logLik(pfilter(gompertz,params=theta.true,Np=10000)))
## loglik.true.est <- bl+log(mean(exp(loglik.true-bl)))
## loglik.true.se <- sd(exp(loglik.true-bl))/sqrt(length(loglik.true))/exp(loglik.true.est-bl)
## #line 736 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"


###################################################
### chunk number 43: multi-mif-plot eval=FALSE
###################################################
## #line 739 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## op <- par(mfrow=c(4,1),mar=c(3,3,0,0),mgp=c(2,1,0),bty='l')
## loglik <- sapply(mf,function(x)conv.rec(x,"loglik"))
## log.r <- sapply(mf,function(x)conv.rec(x,"log.r"))
## log.sigma <- sapply(mf,function(x)conv.rec(x,"log.sigma"))
## log.tau <- sapply(mf,function(x)conv.rec(x,"log.tau"))
## matplot(loglik,type='l',lty=1,xlab="",ylab=expression(log~L),xaxt='n',ylim=max(loglik,na.rm=T)+c(-12,3))
## matplot(log.r,type='l',lty=1,xlab="",ylab=expression(log~r),xaxt='n')
## matplot(log.sigma,type='l',lty=1,xlab="",ylab=expression(log~sigma),xaxt='n')
## matplot(log.tau,type='l',lty=1,xlab="MIF iteration",ylab=expression(log~tau))
## par(op)


###################################################
### chunk number 44: mif-plot
###################################################
#line 754 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
#line 739 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#754#"
op <- par(mfrow=c(4,1),mar=c(3,3,0,0),mgp=c(2,1,0),bty='l')
loglik <- sapply(mf,function(x)conv.rec(x,"loglik"))
log.r <- sapply(mf,function(x)conv.rec(x,"log.r"))
log.sigma <- sapply(mf,function(x)conv.rec(x,"log.sigma"))
log.tau <- sapply(mf,function(x)conv.rec(x,"log.tau"))
matplot(loglik,type='l',lty=1,xlab="",ylab=expression(log~L),xaxt='n',ylim=max(loglik,na.rm=T)+c(-12,3))
matplot(log.r,type='l',lty=1,xlab="",ylab=expression(log~r),xaxt='n')
matplot(log.sigma,type='l',lty=1,xlab="",ylab=expression(log~sigma),xaxt='n')
matplot(log.tau,type='l',lty=1,xlab="MIF iteration",ylab=expression(log~tau))
par(op)
#line 755 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"


###################################################
### chunk number 45: first-mif-results-table
###################################################
#line 764 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
print(results.table)


###################################################
### chunk number 46: gompertz-skeleton-def
###################################################
#line 790 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
gompertz.skel <- function (x, t, params, ...) {
  delta.t <- 1
  r <- exp(params["log.r"])
  K <- exp(params["log.K"])
  X <- x["X"]
  S <- exp(-r*delta.t)
  xnew <- c(X=unname(K^(1-S)*X^S))
  return(xnew)
}


###################################################
### chunk number 47: new-gompertz-pomp-construction
###################################################
#line 803 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
new.gompertz <- pomp(
                     data=data.frame(time=1:200,Y=NA),
                     times="time",
                     rprocess=discrete.time.sim(gompertz.proc.sim),
                     rmeasure=gompertz.meas.sim,
                     dmeasure=gompertz.meas.dens,
                     skeleton.type="map",
                     skeleton=gompertz.skel,
                     t0=0
                     )
coef(new.gompertz) <- theta
coef(new.gompertz,"X.0") <- 0.1
coef(new.gompertz,"log.r") <- log(0.1)
coef(new.gompertz,"log.tau") <- log(0.05)
coef(new.gompertz,"log.sigma") <- log(0.05)
new.gompertz <- simulate(new.gompertz,seed=88737400L)


###################################################
### chunk number 48: gompertz-trajmatch-calc eval=FALSE
###################################################
## #line 827 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## tm <- traj.match(
##                  new.gompertz,
##                  start=coef(new.gompertz),
##                  est=c("log.r","log.K","log.tau","X.0"),
##                  method="Nelder-Mead",
##                  maxit=1000,
##                  reltol=1e-8
##                  )


###################################################
### chunk number 49: gompertz-trajmatch-eval
###################################################
#line 837 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
binary.file <- "gompertz-trajmatch.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
#line 827 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#841#"
tm <- traj.match(
                 new.gompertz,
                 start=coef(new.gompertz),
                 est=c("log.r","log.K","log.tau","X.0"),
                 method="Nelder-Mead",
                 maxit=1000,
                 reltol=1e-8
                 )
#line 842 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  save(new.gompertz,tm,file=binary.file)
}


###################################################
### chunk number 50: trajmatch-plot
###################################################
#line 848 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
op <- par(mfrow=c(1,1),mar=c(3,3,0,0),mgp=c(2,1,0),bty='l')
plot(time(tm),obs(tm,"Y"),xlab="time",ylab=expression(X,Y),type='o')
lines(time(tm),states(tm,"X"),lwd=2)
par(op)


###################################################
### chunk number 51: ricker-map-defn
###################################################
#line 884 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
ricker.sim <- function (x, t, params, delta.t, ...) {
  e <- rnorm(n=1,mean=0,sd=exp(params["log.sigma"])) 
  xnew <- c(
            exp(params["log.r"]+log(x["N"])-x["N"]+e),
            e
            )
  names(xnew) <- c("N","e")
  xnew
}


###################################################
### chunk number 52: ricker-pomp
###################################################
#line 899 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
ricker <- pomp(
               data=data.frame(time=seq(0,50,by=1),y=NA),
               times="time",
               t0=0,
               rprocess=discrete.time.sim(
                 step.fun=ricker.sim
                 ),
               measurement.model=y~pois(lambda=N*exp(log.phi))
               )
coef(ricker) <- c(
                  log.r=3.8,
                  log.sigma=log(0.3),
                  log.phi=log(10),
                  N.0=7,
                  e.0=0
                  )
ricker <- simulate(ricker,seed=73691676L)


###################################################
### chunk number 53: get-ricker
###################################################
#line 922 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
data(ricker)


###################################################
### chunk number 54: probe-list
###################################################
#line 937 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
plist <- list(
              probe.marginal("y",ref=obs(ricker),transform=sqrt),
              probe.acf("y",lags=c(0,1,2,3,4),transform=sqrt),
              probe.nlar("y",lags=c(1,1,1,2),powers=c(1,2,3,1),transform=sqrt)
              )


###################################################
### chunk number 55: first-probe
###################################################
#line 948 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
pb.truth <- probe(ricker,probes=plist,nsim=1000,seed=1066L)
guess <- c(log.r=log(20),log.sigma=log(1),log.phi=log(20),N.0=7,e.0=0)
pb.guess <- probe(ricker,params=guess,probes=plist,nsim=1000,seed=1066L)


###################################################
### chunk number 56: first-probe-plot eval=FALSE
###################################################
## #line 954 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## summary(pb.truth)
## summary(pb.guess)
## plot(pb.truth)
## plot(pb.guess)


###################################################
### chunk number 57: ricker-probe-plot
###################################################
#line 965 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  pb <- probe(ricker,
              probes=list(
                probe.marginal("y",ref=obs(ricker),transform=sqrt),
                probe.acf("y",lags=c(0,1,3),transform=sqrt),
                mean=probe.mean("y",transform=sqrt)
                       ),
              nsim=1000,
              seed=1066L
              )
  png(
      filename="ricker-probe-plot.png",
      width=6,height=6,units='in',res=300
      )
  plot(pb)
  dev.off()


###################################################
### chunk number 58: ricker-probe-match-calc eval=FALSE
###################################################
## #line 994 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## pm <- probe.match(
##                   pb.guess,
##                   est=c("log.r","log.sigma","log.phi"),
##                   method="Nelder-Mead",
##                   maxit=2000,
##                   seed=1066L,
##                   reltol=1e-8,
##                   trace=3
##                   )
## summary(pm)


###################################################
### chunk number 59: ricker-probe.match-eval
###################################################
#line 1011 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
binary.file <- "ricker-probe-match.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
#line 994 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1015#"
pm <- probe.match(
                  pb.guess,
                  est=c("log.r","log.sigma","log.phi"),
                  method="Nelder-Mead",
                  maxit=2000,
                  seed=1066L,
                  reltol=1e-8,
                  trace=3
                  )
summary(pm)
#line 1016 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  save(pm,file=binary.file)
}


###################################################
### chunk number 60: ricker-mif-calc eval=FALSE
###################################################
## #line 1023 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## mf <- mif(
##           ricker,
##           start=guess,
##           Nmif=100,
##           Np=1000,
##           cooling.factor=0.99,
##           var.factor=2,
##           ic.lag=3,
##           max.fail=50,
##           rw.sd=c(log.r=0.1,log.sigma=0.1,log.phi=0.1)
##           )
## mf <- continue(mf,Nmif=500,max.fail=20)


###################################################
### chunk number 61: ricker-mif-eval
###################################################
#line 1038 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
binary.file <- "ricker-mif.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
#line 1023 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1042#"
mf <- mif(
          ricker,
          start=guess,
          Nmif=100,
          Np=1000,
          cooling.factor=0.99,
          var.factor=2,
          ic.lag=3,
          max.fail=50,
          rw.sd=c(log.r=0.1,log.sigma=0.1,log.phi=0.1)
          )
mf <- continue(mf,Nmif=500,max.fail=20)
#line 1043 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  save(mf,file=binary.file)
}


###################################################
### chunk number 62: 
###################################################
#line 1054 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
pf.truth <- pfilter(ricker,Np=1000,max.fail=50,seed=1066L)
pf.guess <- pfilter(ricker,params=guess,Np=1000,max.fail=50,seed=1066L)
pf.mf <- pfilter(mf,Np=1000,seed=1066L)
pf.pm <- pfilter(pm,Np=1000,max.fail=10,seed=1066L)
pb.mf <- probe(mf,nsim=1000,probes=plist,seed=1066L)
res <- rbind(
             cbind(guess=guess,truth=coef(ricker),MLE=coef(mf),PM=coef(pm)),
             loglik=c(
               pf.guess$loglik,
               pf.truth$loglik,
               pf.mf$loglik,
               pf.pm$loglik
               ),
             synth.loglik=c(
               summary(pb.guess)$synth.loglik,
               summary(pb.truth)$synth.loglik,
               summary(pb.mf)$synth.loglik,
               summary(pm)$synth.loglik
               )
             )

print(res,digits=3)


###################################################
### chunk number 63:  eval=FALSE
###################################################
## #line 1090 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## data(gompertz)
## out <- nlf(
##            gompertz,
##            start=theta.guess,
##            est=c("log.K","log.r"),
##            lags=c(1,2)
##            )


###################################################
### chunk number 64: nlf-gompertz-starts eval=FALSE
###################################################
## #line 1105 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## starts <- list()
## for (j in 1:5) { # Pick 5 random starting parameter values
##     sj <- coef(gompertz)
##     sj[c("log.K","log.r")] <- rnorm(n=2,mean=sj[c("log.K","log.r")],sd=0.1)
##     starts[[j]] <- sj
## }


###################################################
### chunk number 65: nlf-gompertz-fits eval=FALSE
###################################################
## #line 1114 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## out <- list()
## for (j in 1:5) { # Do fits. method, trace, and nasymp are explained below   
##   out[[j]] <- nlf(
##                   gompertz,
##                   start=starts[[j]],
##                   est=c("log.K","log.r"),
##                   lags=c(1,2),
##                   seed=7639873, 
##                   method="Nelder-Mead",
##                   trace=4,
##                   nasymp=5000
##                   )  
## }
## fits <- t(sapply(out,function(x)c(x$params[c("log.r","log.K")],value=x$value)))


###################################################
### chunk number 66: 
###################################################
#line 1130 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
binary.file <- "nlf-fits.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
#line 1105 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1134#"
starts <- list()
for (j in 1:5) { # Pick 5 random starting parameter values
    sj <- coef(gompertz)
    sj[c("log.K","log.r")] <- rnorm(n=2,mean=sj[c("log.K","log.r")],sd=0.1)
    starts[[j]] <- sj
}
#line 1135 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
#line 1114 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1135#"
out <- list()
for (j in 1:5) { # Do fits. method, trace, and nasymp are explained below   
  out[[j]] <- nlf(
                  gompertz,
                  start=starts[[j]],
                  est=c("log.K","log.r"),
                  lags=c(1,2),
                  seed=7639873, 
                  method="Nelder-Mead",
                  trace=4,
                  nasymp=5000
                  )  
}
fits <- t(sapply(out,function(x)c(x$params[c("log.r","log.K")],value=x$value)))
#line 1136 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  save(starts,out,fits,file=binary.file)
}


###################################################
### chunk number 67: 
###################################################
#line 1141 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
fits


###################################################
### chunk number 68: nlf-my-pomp
###################################################
#line 1162 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
long.gomp <- simulate(gompertz,times=1:1000)
theta <- coef(long.gomp)


###################################################
### chunk number 69: nlf-lag-test-log.r eval=FALSE
###################################################
## #line 1167 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## lags <- list(1,2,c(1,2),c(2,3))
## log.r.vals <- theta["log.r"]+seq(-0.69,0.69,length=25)
## fvals <- matrix(nrow=25,ncol=4)
## for (j in 1:25) {
##   cat(j,"\n")
##   pars <- theta
##   pars["log.r"] <- log.r.vals[j]
##   for(k in 1:4) {
##     fvals[j,k] <- nlf(
##                       long.gomp,
##                       start=pars,
##                       nasymp=5000,
##                       est=NULL,
##                       lags=lags[[k]],
##                       eval.only=TRUE
##                       )
##   }
## }


###################################################
### chunk number 70: nlf-lag-test-log.K eval=FALSE
###################################################
## #line 1187 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## log.K.vals <- theta["log.K"]+seq(-0.15,0.15,length=25)
## fvals2 <- matrix(nrow=25,ncol=4)
## for (j in 1:25) {
##   cat(j,"\n") 
##   pars <- theta
##   pars["log.K"] <- log.K.vals[j]
##   pars["X.0"] <- exp(log.K.vals[j])
##   for(k in 1:4) {
##     fvals2[j,k] <- nlf(
##                        long.gomp,
##                        start=pars,
##                        nasymp=5000,
##                        est=NULL,
##                        lags=lags[[k]],
##                        eval.only=TRUE
##                        )
##   }
## }


###################################################
### chunk number 71: nlf-lag-tests
###################################################
#line 1207 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
binary.file <- "nlf-lag-tests.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
#line 1162 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1211#"
long.gomp <- simulate(gompertz,times=1:1000)
theta <- coef(long.gomp)
#line 1212 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
#line 1167 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1212#"
lags <- list(1,2,c(1,2),c(2,3))
log.r.vals <- theta["log.r"]+seq(-0.69,0.69,length=25)
fvals <- matrix(nrow=25,ncol=4)
for (j in 1:25) {
  cat(j,"\n")
  pars <- theta
  pars["log.r"] <- log.r.vals[j]
  for(k in 1:4) {
    fvals[j,k] <- nlf(
                      long.gomp,
                      start=pars,
                      nasymp=5000,
                      est=NULL,
                      lags=lags[[k]],
                      eval.only=TRUE
                      )
  }
}
#line 1213 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
#line 1187 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1213#"
log.K.vals <- theta["log.K"]+seq(-0.15,0.15,length=25)
fvals2 <- matrix(nrow=25,ncol=4)
for (j in 1:25) {
  cat(j,"\n") 
  pars <- theta
  pars["log.K"] <- log.K.vals[j]
  pars["X.0"] <- exp(log.K.vals[j])
  for(k in 1:4) {
    fvals2[j,k] <- nlf(
                       long.gomp,
                       start=pars,
                       nasymp=5000,
                       est=NULL,
                       lags=lags[[k]],
                       eval.only=TRUE
                       )
  }
}
#line 1214 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  save(theta,lags,log.r.vals,log.K.vals,fvals,fvals2,file=binary.file)
}


###################################################
### chunk number 72: nlf-gompertz-plot
###################################################
#line 1220 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
fvals <- scale(fvals,center=apply(fvals,2,max),scale=FALSE) 
fvals2 <- scale(fvals2,center=apply(fvals2,2,max),scale=FALSE)
op <- par(mfrow=c(1,2),mar=c(5,5,1,1))
matplot(
        log.r.vals,
        fvals,
        xlab=quote(log~r),
        lty=1,
        col=c("black","green3","red","purple"),
        ylab="NLF objective function",
        type="o",
        pch=16
        )
abline(v=theta["log.r"],col="blue")
legend("bottomleft",legend=c("1","2","1,2","2,3"),col=c("black","green3","red","purple"),lty=1,pch=16,cex=1,bty="n")

matplot(
        log.K.vals,
        fvals2,
        xlab=quote(log~K),
        lty=1,
        col=c("black","green3","red","purple"),
        ylab="NLF objective function",
        type="o",
        pch=16
        )
abline(v=theta["log.K"],col="blue")
par(op)


###################################################
### chunk number 73: nlf-multi-short eval=FALSE
###################################################
## #line 1260 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## nreps <- 100
## ndata <- 60
## fvals <- matrix(nrow=nreps,ncol=length(lags))
## new.pomp <- simulate(gompertz,times=1:ndata,nsim=nreps,seed=NULL) # nreps simulated data sets 
## for (j in 1:nreps) {
##   for (k in seq_along(lags)) {
##     fvals[j,k] <- nlf(
##                       new.pomp[[j]], 
##                       start=coef(gompertz), 
##                       nasymp=5000, 
##                       est=NULL,
##                       lags=lags[[k]],
##                       eval.only=TRUE
##                       ) 
##   }
## }
## fvals <- exp(fvals/ndata)


###################################################
### chunk number 74: nlf-multi-short-eval
###################################################
#line 1279 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
binary.file <- "nlf-multi-short.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
#line 1260 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1283#"
nreps <- 100
ndata <- 60
fvals <- matrix(nrow=nreps,ncol=length(lags))
new.pomp <- simulate(gompertz,times=1:ndata,nsim=nreps,seed=NULL) # nreps simulated data sets 
for (j in 1:nreps) {
  for (k in seq_along(lags)) {
    fvals[j,k] <- nlf(
                      new.pomp[[j]], 
                      start=coef(gompertz), 
                      nasymp=5000, 
                      est=NULL,
                      lags=lags[[k]],
                      eval.only=TRUE
                      ) 
  }
}
fvals <- exp(fvals/ndata)
#line 1284 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  save(lags,nreps,ndata,fvals,file=binary.file)
}


###################################################
### chunk number 75: 
###################################################
#line 1290 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
apply(fvals,2,function(x)sd(x)/mean(x))


###################################################
### chunk number 76: nlf-fit-from-truth eval=FALSE
###################################################
## #line 1294 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## true.fit <- nlf(
##                 gompertz,
##                 start=coef(gompertz),
##                 est=c("log.K","log.r"),
##                 lags=2,
##                 seed=7639873, 
##                 method="Nelder-Mead",
##                 trace=4,
##                 nasymp=5000
##                 )


###################################################
### chunk number 77: nlf-fit-from-truth-eval
###################################################
#line 1306 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
binary.file <- "nlf-fit-from-truth.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
#line 1294 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1310#"
true.fit <- nlf(
                gompertz,
                start=coef(gompertz),
                est=c("log.K","log.r"),
                lags=2,
                seed=7639873, 
                method="Nelder-Mead",
                trace=4,
                nasymp=5000
                )
#line 1311 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  save(true.fit,file=binary.file)
}


###################################################
### chunk number 78: 
###################################################
#line 1329 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
set.seed(32329L)


###################################################
### chunk number 79: nlf-boot eval=FALSE
###################################################
## #line 1332 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## lags <- 2
## ndata <- length(obs(gompertz))
## nboot <- ndata-max(lags)
## nreps <- 100
## pars <- matrix(0,nreps,2)
## bootsamp <- replicate(n=nreps,sample(nboot,replace=TRUE))
## for (j in seq_len(nreps)) {
##   fit <- nlf(
##              gompertz,
##              start=coef(gompertz),
##              est=c("log.K","log.r"),
##              lags=lags,
##              seed=7639873, 
##              bootstrap=TRUE, 
##              bootsamp=bootsamp[,j],
##              skip.se=TRUE, 
##              method="Nelder-Mead",
##              trace=4,
##              nasymp=5000
##              )
##    pars[j,] <- fit$params[c("log.r","log.K")]
## }
## colnames(pars) <- c("log.r","log.K")


###################################################
### chunk number 80: nlf-boot-eval
###################################################
#line 1357 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
binary.file <- "nlf-boot.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
#line 1332 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1361#"
lags <- 2
ndata <- length(obs(gompertz))
nboot <- ndata-max(lags)
nreps <- 100
pars <- matrix(0,nreps,2)
bootsamp <- replicate(n=nreps,sample(nboot,replace=TRUE))
for (j in seq_len(nreps)) {
  fit <- nlf(
             gompertz,
             start=coef(gompertz),
             est=c("log.K","log.r"),
             lags=lags,
             seed=7639873, 
             bootstrap=TRUE, 
             bootsamp=bootsamp[,j],
             skip.se=TRUE, 
             method="Nelder-Mead",
             trace=4,
             nasymp=5000
             )
   pars[j,] <- fit$params[c("log.r","log.K")]
}
colnames(pars) <- c("log.r","log.K")
#line 1362 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  save(pars,file=binary.file)
}


###################################################
### chunk number 81: 
###################################################
#line 1366 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
apply(pars,2,sd)


###################################################
### chunk number 82: block-bootsamp eval=FALSE
###################################################
## #line 1376 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## bootsamp <- replicate(n=nreps,sample(nboot,size=floor(nboot/3),replace=TRUE))
## bootsamp <- rbind(bootsamp,bootsamp+1,bootsamp+2)


###################################################
### chunk number 83: nlf-block-boot eval=FALSE
###################################################
## #line 1381 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## lags <- 2
## ndata <- length(obs(gompertz))
## nboot <- ndata-max(lags)
## nreps <- 100
## pars <- matrix(0,nreps,2)
## bootsamp <- replicate(
##                       n=nreps,
##                       sample(nboot-2,size=floor(nboot/3),replace=TRUE)
##                       )
## bootsamp <- rbind(bootsamp,bootsamp+1,bootsamp+2)
## for (j in seq_len(nreps)) {
##   fit <- nlf(
##              gompertz,
##              start=coef(gompertz),
##              est=c("log.K","log.r"),
##              lags=lags,
##              seed=7639873, 
##              bootstrap=TRUE, 
##              bootsamp=bootsamp[,j],
##              skip.se=TRUE, 
##              method="Nelder-Mead",
##              trace=4,
##              nasymp=5000
##              )
##    pars[j,] <- fit$params[c("log.r","log.K")]
## }
## colnames(pars) <- c("log.r","log.K")


###################################################
### chunk number 84: nlf-block-boot-eval
###################################################
#line 1411 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
binary.file <- "nlf-block-boot.rda"
if (file.exists(binary.file)) {
  load(binary.file)
} else {
#line 1381 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw#from line#1415#"
lags <- 2
ndata <- length(obs(gompertz))
nboot <- ndata-max(lags)
nreps <- 100
pars <- matrix(0,nreps,2)
bootsamp <- replicate(
                      n=nreps,
                      sample(nboot-2,size=floor(nboot/3),replace=TRUE)
                      )
bootsamp <- rbind(bootsamp,bootsamp+1,bootsamp+2)
for (j in seq_len(nreps)) {
  fit <- nlf(
             gompertz,
             start=coef(gompertz),
             est=c("log.K","log.r"),
             lags=lags,
             seed=7639873, 
             bootstrap=TRUE, 
             bootsamp=bootsamp[,j],
             skip.se=TRUE, 
             method="Nelder-Mead",
             trace=4,
             nasymp=5000
             )
   pars[j,] <- fit$params[c("log.r","log.K")]
}
colnames(pars) <- c("log.r","log.K")
#line 1416 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
  save(pars,file=binary.file)
}


###################################################
### chunk number 85: sir-proc-sim-def
###################################################
#line 1547 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
sir.proc.sim <- function (x, t, params, delta.t, ...) {
  ## untransform the parameters
  N <- exp(params["N"])             # population size
  gamma <- exp(params["gamma"])     # recovery rate
  mu <- exp(params["mu"])           # birth rate = death rate
  beta <- exp(params["beta"])       # contact rate
  foi <- beta*x["I"]/N                       # the force of infection
  trans <- c(
             rpois(n=1,lambda=mu*N*delta.t), # births are assumed to be Poisson
             reulermultinom(n=1,size=x["S"],rate=c(foi,mu),dt=delta.t), # exits from S
             reulermultinom(n=1,size=x["I"],rate=c(gamma,mu),dt=delta.t), # exits from I
             reulermultinom(n=1,size=x["R"],rate=c(mu),dt=delta.t)        # exits from R
             )
  ## now connect the compartments
  x[c("S","I","R","cases")]+c(
      trans[1]-trans[2]-trans[3],
      trans[2]-trans[4]-trans[5],
      trans[4]-trans[6],
      trans[4]                          # accumulate the recoveries
    )
}


###################################################
### chunk number 86: sir-pomp-def
###################################################
#line 1590 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
simulate(
         pomp(
              data=data.frame(
                time=seq(1/52,15,by=1/52),
                reports=NA
                ),
              times="time",
              t0=0,
              rprocess=euler.sim(
                step.fun=sir.proc.sim,
                delta.t=1/52/20
                ),
              measurement.model=reports~binom(size=cases,prob=exp(rho)),
              initializer=function(params, t0, ic.pars, comp.names, ...){
                x0 <- c(S=0,I=0,R=0,cases=0)
                N <- exp(params["N"])
                fracs <- exp(params[ic.pars])
                x0[comp.names] <- round(N*fracs/sum(fracs))
                x0
              },
              zeronames=c("cases"), # zero this variable after each obsvn.
              ic.pars=c("S0","I0","R0"), # initial condition parameters
              comp.names=c("S","I","R") # names of the compartments
              ),
         params=log(
           c(
             N=50000,
             beta=60,gamma=8,mu=1/50,
             rho=0.6,
             S0=8/60,I0=0.002,R0=1-8/60-0.001
             )
           ),
         seed=677573454L
         ) -> sir


###################################################
### chunk number 87: sir-pomp-def-with-skel eval=FALSE
###################################################
## #line 1626 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
## simulate(
##          pomp(
##               data=data.frame(
##                 time=seq(1/52,15,by=1/52),
##                 reports=NA
##                 ),
##               times="time",
##               t0=0,
##               rprocess=euler.sim(
##                 step.fun=sir.proc.sim,
##                 delta.t=1/52/20
##                 ),
##               skeleton.type="vectorfield",
##               skeleton=function(t,x,params,...){
##                 N <- exp(params["N"])             # population size
##                 gamma <- exp(params["gamma"])     # recovery rate
##                 mu <- exp(params["mu"])           # birth rate = death rate
##                 beta <- exp(params["beta"])       # contact rate
##                 foi <- beta*x["I"]/N # the force of infection
##                 terms <- c(
##                            mu*N,               # births
##                            x["S"]*c(foi,mu),   # exits from S
##                            x["I"]*c(gamma,mu), # exits from I
##                            x["R"]*mu           # exits from R
##                            )
##                 ## now put the equations together:
##                 c(
##                   ## approx. number of new infections/week:
##                   cases=unname((terms[2]-terms[4]-terms[5])*gamma/52),
##                   S=unname(terms[1]-terms[2]-terms[3]), # dS/dt
##                   I=unname(terms[2]-terms[4]-terms[5]), # dI/dt
##                   R=unname(terms[4]-terms[6])           # dR/dt
##                   )
##               },
##               measurement.model=reports~binom(size=cases,prob=exp(rho)),
##               initializer=function(params, t0, ic.pars, comp.names, ...){
##                 x0 <- c(S=0,I=0,R=0,cases=0)
##                 N <- exp(params["N"])
##                 fracs <- exp(params[ic.pars])
##                 x0[comp.names] <- round(N*fracs/sum(fracs))
##                 x0
##               },
##               zeronames=c("cases"), # zero this variable after each obsvn.
##               ic.pars=c("S0","I0","R0"), # initial condition parameters
##               comp.names=c("S","I","R") # names of the compartments
##               ),
##          params=log(
##            c(
##              N=50000,
##              beta=60,gamma=8,mu=1/50,
##              rho=0.6,
##              S0=8/60,I0=0.002,R0=1-8/60-0.001
##              )
##            ),
##          seed=677573454L
##          ) -> sir


###################################################
### chunk number 88: sir-plot
###################################################
#line 1690 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
plot(sir)


###################################################
### chunk number 89: seas-basis
###################################################
#line 1716 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
tbasis <- seq(0,20,by=1/52)
basis <- periodic.bspline.basis(tbasis,nbasis=3,degree=2,period=1,names="seas%d")


###################################################
### chunk number 90: complex-sir-def
###################################################
#line 1740 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
complex.sir.proc.sim <- function (x, t, params, delta.t, covars, ...) {
  ## untransform the parameters
  N <- exp(params["N"])                 # population size
  gamma <- exp(params["gamma"])         # recovery rate
  mu <- exp(params["mu"])               # birth rate = death rate
  iota <- exp(params["iota"])           # import rate
  b <- params[c("b1","b2","b3")] # contact-rate coefficients
  beta <- exp(b%*%covars)       # flexible seasonality
  beta.sd <- exp(params["beta.sd"])     # extrademographic noise intensity
  beta.var <- beta.sd*beta.sd
  if (beta.var > 0) 
    dW <- rgamma(n=1,shape=delta.t/beta.var,scale=beta.var)
  else
    dW <- delta.t
  foi <- (beta*x["I"]/N+iota)*dW/delta.t # the force of infection
  trans <- c(
             rpois(n=1,lambda=mu*N*delta.t), # births are assumed to be Poisson
             reulermultinom(n=1,size=x["S"],rate=c(foi,mu),dt=delta.t), # exits from S
             reulermultinom(n=1,size=x["I"],rate=c(gamma,mu),dt=delta.t), # exits from I
             reulermultinom(n=1,size=x["R"],rate=c(mu),dt=delta.t)        # exits from R
             )
  ## now connect the compartments
  x[c("S","I","R","cases","W")]+
    c(
      trans[1]-trans[2]-trans[3],
      trans[2]-trans[4]-trans[5],
      trans[4]-trans[6],
      trans[4],                   # accumulate the recoveries
      (dW-delta.t)/beta.sd        # mean = 0, var = delta.t
    )
}

simulate(
         pomp(
              sir,
              tcovar=tbasis,
              covar=basis,
              rprocess=euler.sim(
                complex.sir.proc.sim,
                delta.t=1/52/20
                ),
              initializer=function(params, t0, ic.pars, comp.names, ...){
                x0 <- c(S=0,I=0,R=0,cases=0,W=0)
                N <- exp(params["N"])
                fracs <- exp(params[ic.pars])
                x0[comp.names] <- round(N*fracs/sum(fracs))
                x0
              }
              ),
         params=log(
           c(
             N=50000,
             b1=60,b2=10,b3=110,
             gamma=8,mu=1/50,
             rho=0.6,
             iota=0.01,beta.sd=0.1,
             S0=8/60,I0=0.002,R0=1-8/60-0.001
             )
           ),
         seed=8274355L
         ) -> complex.sir


###################################################
### chunk number 91: seas-basis-plot
###################################################
#line 1807 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
op <- par(mar=c(5,5,1,5))
matplot(tbasis,basis,xlim=c(0,2),type='l',lwd=2,bty='u',
        lty=1,col=c("red","blue","orange"),xlab="time (yr)",
        ylab=quote("basis functions"~list(s[1],s[2],s[3])))
bb <- coef(complex.sir,c("b1","b2","b3"))
plot.window(c(0,2),c(0,1)*exp(max(bb)))
lines(tbasis,exp(basis%*%bb),col="black",lwd=3,lty=1)
lines(tbasis,exp(basis%*%bb),col="white",lwd=2.5,lty="23")
axis(side=4)
mtext(
      side=4,
      line=2,
      text=bquote(
        beta==exp(.(b1)*s[1]+.(b2)*s[2]+.(b3)*s[3]),
        where=as.list(exp(coef(complex.sir)))
        )
      )
par(op)


###################################################
### chunk number 92: complex-sir-plot
###################################################
#line 1837 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
plot(complex.sir)


###################################################
### chunk number 93: restore-opts
###################################################
#line 1853 "d:/Rcompile/CRANpkg/local/2.12/pomp/inst/doc/intro_to_pomp.Rnw"
options(glop)


